#include <string>
#include <iostream>
using namespace std;

/**
* The SquarePeg class.
* This is the Target class.
*/
class SquarePeg
{
   public:
      virtual void insertIntoSquareHole(string str){
         cout << "SquarePeg insertIntoSquareHole(): " <<  str;}
};

/**
* The RoundPeg class.
* This is the Adaptee class.
*/
class RoundPeg {
  public:
         void insertIntoRoundHole(string str) {
              cout << "RoundPeg insertIntoRoundHole(): " << str;}
};

/**
* The RoundToSquarePegAdapter class.
* This is the Adapter class.
* It adapts a RoundPeg to a SquarePeg.
* Its interface is that of a SquarePeg.
*/
class RoundToSquarePegAdapter: public SquarePeg {
  private:
          RoundPeg roundPeg; 
  public:
          RoundToSquarePegAdapter(RoundPeg peg) {
             //the roundPeg is plugged into the adapter
             roundPeg = peg;}
          void insertIntoSquareHole(string str) {
             //the roundPeg can now be inserted in the same manner as a squarePeg!
             roundPeg.insertIntoRoundHole(str);}
};


int main(int argc, char *argv[])
{
  
    // Create some pegs.
    RoundPeg *roundPeg = new RoundPeg();
    SquarePeg *squarePeg = new SquarePeg();

    // Do an insert using the square peg.
    squarePeg->insertIntoSquareHole("Inserting square peg...\n");

    // Now we'd like to do an insert using the round peg.
    // But this client only understands the insert()
    // method of pegs, not a insertIntoHole() method.
    // The solution: create an adapter that adapts
    // a square peg to a round peg!

    RoundToSquarePegAdapter *adapter = new RoundToSquarePegAdapter(*roundPeg);
    adapter->insertIntoSquareHole("Inserting round peg...\n");

    SquarePeg *sq_pegs[2] = {new SquarePeg(), 
                             new RoundToSquarePegAdapter(*roundPeg)};
    sq_pegs[0]->insertIntoSquareHole("Inserting square peg [0]...\n");
    sq_pegs[1]->insertIntoSquareHole("Inserting square peg [1]...\n");
    
    system("PAUSE");
    return EXIT_SUCCESS;
}
